<?php

if ( is_admin() ) {

	/* post & page */
	function wcnn_add_metabox() {

		$post_types = get_post_types( array( 'public' => true ) );

		foreach ( $post_types as $key ) {

			if ( $key == 'attachment' ) {
				continue;
			}

			add_meta_box( 'wcnn', 'Noindex Nofollow', 'wcnn_print_metabox', $post_types[$key], 'side', 'default' );

		}

	}
	add_action( 'admin_menu', 'wcnn_add_metabox' );

	function wcnn_print_metabox() {

		global $post;
		wp_nonce_field( wp_create_nonce( __FILE__ ), 'wcnn_nonce' );
		$wcnn_meta = get_post_meta( $post->ID, 'wcnn', true );

		?>
		<label><input type="checkbox" name="wcnn[noindex_post]" value="1" <?php if ( isset( $wcnn_meta['noindex_post'] ) ) { echo 'checked'; } ?>> noindex</label>
		<label><input type="checkbox" name="wcnn[nofollow_post]" value="1" <?php if ( isset( $wcnn_meta['nofollow_post'] ) ) { echo 'checked'; } ?>> nofollow</label>
		<?php

	}

	function wcnn_save_postmeta( $post_id ) {

		if ( ! isset( $_POST['wcnn_nonce'] ) || ! wp_verify_nonce( $_POST['wcnn_nonce'], wp_create_nonce( __FILE__ ) ) ) {
			return $post_id;
		}

		if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE ) {
			return $post_id;
		}

		if ( ! current_user_can( 'edit_post', $post_id ) ) {
			return $post_id;
		}

		if ( isset ( $_POST['wcnn'] ) ) {

			if ( isset( $_POST['wcnn']['noindex_post'] ) ) {
				$wcnn['noindex_post'] = true;
			}

			if ( isset( $_POST['wcnn']['nofollow_post'] ) ) {
				$wcnn['nofollow_post'] = true;
			}

			update_post_meta( $post_id, 'wcnn', $wcnn );

		} else {
			delete_post_meta( $post_id, 'wcnn' );
		}

	}
	add_action( 'save_post', 'wcnn_save_postmeta' );

	/* taxonomy */
	function wcnn_term_fields( $term ) {

		$wcnn_term_meta = get_term_meta( $term->term_id, 'wcnn', true );
		$wcnn_term_name = esc_attr( $term->name );
		$wcnn_term_link = get_term_link( $term, $term->taxonomy );

		if ( ! isset( $wcnn_term_meta['noindex_archives'] ) ) {

			if ( ! is_array( $wcnn_term_meta ) ) {
				$wcnn_term_meta = array();
			}

			$wcnn_term_meta['noindex_archives'] = 0;

		}

	?>
	<tr class="form-field">
		<th><label for="wcnn_noindex_archives">noindex化</label></th>
		<td>
		<label><input type="checkbox" name="wcnn[noindex_archives]" value="1" <?php if ( $wcnn_term_meta['noindex_archives'] ) { echo 'checked'; } ?> /> 有効</label>
		<p class="description">「<?php echo "<a href=\"$wcnn_term_link\" target=\"_blank\">$wcnn_term_name</a>"; ?>」のアーカイブページをnoindexに設定する場合にチェック</p>
		</td>
	</tr>
	<?php
	}
	add_action ( 'edit_category_form_fields', 'wcnn_term_fields' );
	add_action ( 'edit_tag_form_fields', 'wcnn_term_fields');

	if ( $taxonomies = get_taxonomies( array( 'public' => true, '_builtin' => false ) ) ) {
		foreach ( $taxonomies  as $taxonomy ) {
			add_action ( $taxonomy . '_edit_form_fields', 'wcnn_term_fields');
		}
	}

	function wcnn_save_term_meta( $term_id ) {

		if ( isset( $_POST['wcnn'] ) ) {

			foreach ( array_keys( $_POST['wcnn'] ) as $key ) {

				if ( isset( $_POST['wcnn'][$key] ) ) {

					$wcnn_term_meta[$key] = true;

				}

			}

			update_term_meta( $term_id, 'wcnn', $wcnn_term_meta );

		} else if ( ! isset( $_POST['wcnn'] ) && get_term_meta( $term_id, 'wcnn', true ) ) {

			delete_term_meta( $term_id, 'wcnn' );

		}

	}
	add_action ( 'edited_term', 'wcnn_save_term_meta' );

	function wcnn_get_all_post() {

		$args = array(
			'orderby'        => 'ID',
			'order'          => 'ASC',
			'meta_key'       => 'wcnn',
			'post_type'      => 'any',
			'posts_per_page' => -1
		);

		$wcnn_allposts = get_posts( $args );
		$wcnn_allposts_id = array();

		foreach ( $wcnn_allposts as $wcnn_post ) {
			array_push( $wcnn_allposts_id, $wcnn_post->ID );
		}

		wp_reset_postdata();

		return $wcnn_allposts_id;

	}

	function wcnn_get_all_term() {

		$taxonomies = array(
			'category',
			'post_tag'
		);

		if ( $custom_taxonomies = get_taxonomies( array( 'public' => true, '_builtin' => false ) ) ) {
			foreach ( $custom_taxonomies  as $custom_taxonomy ) {
				array_push( $taxonomies, $custom_taxonomy );
			}
		}

		$args = array(
			'orderby'       => 'id',
			'order'         => 'ASC',
			'fields'        => 'ids',
			'hide_empty'    => false,
			'hierarchical'  => false,
			'pad_counts'    => false,
			'cache_domain'  => 'core'
		);

		$all_terms = get_terms( $taxonomies, $args );
		$all_term_meta = array();

		foreach ( $all_terms as $term_id ) {

			if ( get_term_meta( $term_id, 'wcnn', true ) ) {
				array_push( $all_term_meta, $term_id );
			}

		}

		return $all_term_meta;

	}

}

function wcnn_add_noindex() {

	$opt = get_option( 'wcnn_options' );
	$noindex = false;
	$nofollow = false;

	if ( is_singular() ) {

		global $post;

		if ( $wcnn_meta = get_post_meta( $post->ID, 'wcnn', true ) ) {

			if ( isset( $wcnn_meta['noindex_post'] ) ) {
				$noindex = true;
			}

			if ( isset( $wcnn_meta['nofollow_post'] ) ) {
				$nofollow = true;
			}

		}

	} else if ( is_category() || is_tag() || is_tax() ) {

		$term = get_queried_object();

		if ( is_category() && isset( $opt['noindex_category'] ) && $opt['noindex_category'] ) {

			$noindex = true;

		} else if ( is_tag() && isset( $opt['noindex_post_tag'] ) && $opt['noindex_post_tag'] ) {

			$noindex = true;

		} else if ( is_tax() && isset( $opt['noindex_' . $term->taxonomy] ) && $opt['noindex_' . $term->taxonomy] ) {

			$noindex = true;

		} else if ( $wcnn_term_meta = get_term_meta( $term->term_id, 'wcnn', true ) ) {

			if ( $wcnn_term_meta["noindex_archives"] ) {
				$noindex = true;
			}

		}

	} else if ( is_search() && isset( $opt['noindex_search'] ) && $opt['noindex_search'] ) {

		$noindex = true;

	} else if ( is_date() && isset( $opt['noindex_date'] ) && $opt['noindex_date'] ) {

		$noindex = true;

	} else if ( is_author() && isset( $opt['noindex_author'] ) && $opt['noindex_author'] ) {

		$noindex = true;

	} else if ( is_404() && isset( $opt['noindex_404'] ) && $opt['noindex_404'] ) {

		$noindex = true;

	}

	if ( is_singular() ) {

		$opt_key = 'noindex_' . get_post_type();

		if ( isset( $opt[$opt_key] ) && $opt[$opt_key] ) {
			$noindex = true;
		}

	}

	if ( is_paged() && isset( $opt['noindex_paged'] ) && $opt['noindex_paged'] ) {
		$noindex = true;
	}

	if ( $noindex && $nofollow ) {

		echo "<meta name=\"robots\" content=\"noindex, nofollow\">\n";

	} else if ( $noindex && ! $nofollow ) {

		echo "<meta name=\"robots\" content=\"noindex, follow\">\n";

	} else if ( ! $noindex && $nofollow ) {

		echo "<meta name=\"robots\" content=\"index, nofollow\">\n";

	}

}
add_action( 'wp_head', 'wcnn_add_noindex', 1 );

?>
